<?php
/**
 * WHMCS GridCP
 */



require_once __DIR__ . '/lib/clientservice.php';
require_once __DIR__ . '/lib/CaseInsensitiveArray.php';

use Illuminate\Database\Capsule\Manager as Capsule;

if (!defined("WHMCS")) {
    die("This file cannot be accessed directly");
}

// Require any libraries needed for the module to function.
// require_once __DIR__ . '/path/to/library/loader.php';
//
// Also, perform any initialization required by the service's library.

/**
 * Define module related meta data.
 *
 * Values returned here are used to determine module related abilities and
 * settings.
 *
 * @see https://developers.whmcs.com/provisioning-modules/meta-data-params/
 *
 * @return array
 */
function gridcp_MetaData()
{
    return array(
        'DisplayName' => 'GridCP',
        'APIVersion' => '1.0', // Use API Version 1.1
        'RequiresServer' => true, // Set true if module requires a server to work
        'DefaultNonSSLPort' => '80', // Default Non-SSL Connection Port
        'DefaultSSLPort' => '443', // Default SSL Connection Port
        'ServiceSingleSignOnLabel' => 'Login to Panel as User',
        'AdminSingleSignOnLabel' => 'Login to Panel as Admin',
    );
}

/*function initConfigOption()
{

}*/

/**
 * Define product configuration options.
 *
 * The values you return here define the configuration options that are
 * presented to a user when configuring a product for use with the module. These
 * values are then made available in all module function calls with the key name
 * configoptionX - with X being the index number of the field from 1 to 24.
 *
 * You can specify up to 24 parameters, with field types:
 * * text
 * * password
 * * yesno
 * * dropdown
 * * radio
 * * textarea
 *
 * Examples of each and their possible configuration parameters are provided in
 * this sample function.
 *
 * @see https://developers.whmcs.com/provisioning-modules/config-options/
 *
 * @return array
 */
function gridcp_ConfigOptions()
{

    $server_array = array();
    foreach ( Capsule::table( 'tblservers' )->where( 'type', 'gridcp' )->get() as $row ) {
        $server_array[] = $row->id . " - " . $row->name;
    }

    $aux = array();
    if(!isset($_POST['id'])){
        $data = Capsule::table('tblproducts')->where('servertype', 'gridcp')->where('id', $_GET['id'])->get();
    }else{
        $data = Capsule::table('tblproducts')->where('servertype', 'gridcp')->where('id', $_POST['id'])->get();
    }
    $server_select1="";
    if(is_object($data[0]) && count($data[0]) > 0) {
        $aux[0] = $data[0]->configoption1;
        $id_server=trim(explode('-',$aux[0])[0]);

        $server_select=array();
        foreach ( Capsule::table( 'tblservers' )->where( 'type', 'gridcp' )->where('id', $id_server)->get() as $row ) {
            $server_select['id'] = $row->id;
            $server_select['hostname'] = $row->hostname;
            $server_select['username'] = $row->username;
            $server_select['password'] = $row->password;

            $server_select1 .= "{$row->id},{$row->hostname}";
        }
    }

    $nodes='';
    $plans='';
    $floatgroups='';
    $nodegroups='';
    $virtualization_type='';
    if(!empty($server_select)) {
        $clientApi = new clientApi($server_select['hostname'], $server_select['username'], decrypt($server_select['password']), 80, true);
        if(!$clientApi->login())
            throw new Exception('Error connecting to the GridCP panel (check server configuration)');
        $result = $clientApi->get('api/public/plan/kvm')['message'];
        for ($i = 0; $i < count($result); $i++) {
            $plans .= "{$result[$i]['id']} | {$result[$i]['text']}";
            if (count($result) - 1 > $i)
                $plans .= ',';
        }

        $result = $clientApi->get('api/public/floatgroup')['message'];
        for ($i = 0; $i < count($result); $i++) {
            $floatgroups .= "{$result[$i]['id']} | {$result[$i]['name']}";
            if (count($result) - 1 > $i)
                $floatgroups .= ',';
        }

        $result = $clientApi->get('api/public/nodegroup')['message'];
        for ($i = 0; $i < count($result); $i++) {
            $nodegroups .= "{$result[$i]['id']} | {$result[$i]['name']}";
            if (count($result) - 1 > $i)
                $nodegroups .= ',';
        }

        $result = $clientApi->get('api/public/virtualizationtype')['message'];
        foreach ($result as $key => $value){
            $virtualization_type .= "{$result[$key]},";
        }
    }

    $server_list = implode( ",", $server_array );

    return array(
        "Server" => array( "Type" => "dropdown", "Options" => "$server_list" ),
        "Virtualization Type" => array( "Type" => "dropdown", "Options" => $virtualization_type ),
        "Plans" => array( "Type" => "dropdown", "Options" => $plans ),
        "Float Group" => array( "Type" => "dropdown", "Options" => $floatgroups ),
        'Username Prefix' => array(
            'Type' => 'text',
            'Size' => '30',
            'Default' => '',
            'Description' => 'Username Prefix',
        ),
        "Node Group" => array( "Type" => "dropdown", "Options" => $nodegroups ),
    );
}

/**
 * Provision a new instance of a product/service.
 *
 *
 * @param array $params common module parameters
 *
 * @see https://developers.whmcs.com/provisioning-modules/module-parameters/
 *
 * @return string "success" or an error message
 */
function gridcp_CreateAccount(array $params)
{
    try {
        $clientApi = new clientApi($params['serverhostname'], $params['serverusername'], $params['serverpassword'], 80, true);
        if(!$clientApi->login())
            throw new Exception('Error connecting to the GridCP panel (check server configuration)');
        if(empty($clientApi->token)) {
            throw new Exception('Error token.');
        }
        else
        {
            $username = $params['configoption5'] . $params['clientsdetails']['userid'];
            $customField = $params['customfields'];
            $params_post = array();
            $params_post['firstname'] = $params['clientsdetails']['firstname'];
            $params_post['lastname'] = $params['clientsdetails']['lastname'];
            $params_post['email'] = $params['clientsdetails']['email'];
            $params_post['username'] = $username;
            $params_post['password'] = $params['password'];
            $result = $clientApi->post('api/public/user', $params_post);

            $params_post = array();
            $params_post['id_plan'] = trim(explode('|', $params['configoption3'])[0]);
            $params_post['username'] = $username;
            $params_post['hostname'] = $params['domain'];
            $params_post['external_id'] = $customField['vserverid'];
            $params_post['id_float_group'] = trim(explode('|', $params['configoption4'])[0]);
            $params_post['id_node_group'] = trim(explode('|', $params['configoption6'])[0]);
            $params_post['step'] = 1;

            if(!empty($params['configoptions']['Location']))
                $params_post['id_node_group']=$params['configoptions']['Location'];
            if(!empty($params['configoptions']['Location2']))
                $params_post['id_node_group2']=$params['configoptions']['Location2'];
            if(!empty($params['configoptions']['Location3']))
                $params_post['id_node_group3']=$params['configoptions']['Location3'];
            if(!empty($params['configoptions']['Traffic']))
                $params_post['traffic']=$params['configoptions']['Traffic'];

            $result = $clientApi->post('api/public/vps', $params_post);
            if ($result['success']) {
                $return = 'success';

                if($result['message'] > 0 )
                    $id_vps = $result['message'];
                else
                    throw new Exception('Error recovering the id of the VPS. '.$result['message']);
            }
            else
                throw new Exception('Error creating VPS. '.$result['message']);

            $result = $clientApi->get("api/public/getip/{$id_vps}");
            if ($result['success']) {
                $return = 'success';
                $mainip = $result['message'];
            }

            ## Update the username field
            Capsule::table( 'tblhosting' )
                ->where( 'id', $params['serviceid'] )
                ->update(
                    [
                        'username' => $username,
                    ]
                );
            $vserverid=$id_vps;
               if(empty($vserverid))
                    $vserverid = $customField['vserverid'];
            $res = Capsule::table( 'tblcustomfields' )->where( 'relid', $params['pid'] )->where( 'fieldname', 'vserverid' )->first();

            if($res) {
                $fieldValue = Capsule::table('tblcustomfieldsvalues')->where('relid', $params['serviceid'])->where('fieldid', $res->id)->first();

                if ($fieldValue) {
                    if ($fieldValue->value !== $vserverid) {
                        Capsule::table('tblcustomfieldsvalues')
                            ->where('relid', $params['serviceid'])
                            ->where('fieldid', $res->id)
                            ->update(
                                [
                                    'value' => $vserverid,
                                ]
                            );
                    }
                } else {
                    Capsule::table('tblcustomfieldsvalues')
                        ->insert(
                            [
                                'relid' => $params['serviceid'],
                                'fieldid' => $res->id,
                                'value' => $vserverid,
                            ]
                        );
                }
            }

            Capsule::table( 'tblhosting' )
                ->where( 'id', $params['serviceid'] )
                ->update(
                    [
                        'dedicatedip' => $mainip,
                    ]
                );

            $params_post = array();
            $params_post['id_vps'] = $vserverid;
            $params_post['step'] = 2;
            $result = $clientApi->post('api/public/vps', $params_post);
            if($result['success'])
                $return = 'success';
        }
        $clientApi->close();
        logModuleCall(
            'gridcp',
            __FUNCTION__,
            $params,
            $params_post

        );
        return $return;

    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        $clientApi->close();
        logModuleCall(
            'gridcp',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        return $e->getMessage();
    }
}

/**
 * Suspend an instance of a product/service.
 *
 * @param array $params common module parameters
 *
 * @see https://developers.whmcs.com/provisioning-modules/module-parameters/
 *
 * @return string "success" or an error message
 */
function gridcp_SuspendAccount(array $params)
{
    try {
        $clientApi = new clientApi($params['serverhostname'], $params['serverusername'], $params['serverpassword'], 80, true);
        if(!$clientApi->login())
            throw new Exception('Error connecting to the GridCP panel (check server configuration)');
        if(empty($clientApi->token))
            throw new Exception('Error token.');

        $res = Capsule::table( 'tblcustomfields' )->where( 'relid', $params['pid'] )->where( 'fieldname', 'vserverid' )->first();
        $fieldValue = Capsule::table('tblcustomfieldsvalues')->where('relid', $params['serviceid'])->where('fieldid', $res->id)->first();
        $return = $clientApi->post("api/public/vps/suspend/{$fieldValue->value}");
        logModuleCall(
            'gridcp',
            __FUNCTION__,
            $params,
            $return
        );
        if($return['success'])
            return 'success';
        else
            return $return['message'];

    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'gridcp',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        return $e->getMessage();
    }
}

/**
 * Un-suspend instance of a product/service.
 *
 * @param array $params common module parameters
 *
 * @see https://developers.whmcs.com/provisioning-modules/module-parameters/
 *
 * @return string "success" or an error message
 */
function gridcp_UnsuspendAccount(array $params)
{
    try {
        $clientApi = new clientApi($params['serverhostname'], $params['serverusername'], $params['serverpassword'], 80, true);
        if(!$clientApi->login())
            throw new Exception('Error connecting to the GridCP panel (check server configuration)');
        if(empty($clientApi->token))
            throw new Exception('Error token.');

        $res = Capsule::table( 'tblcustomfields' )->where( 'relid', $params['pid'] )->where( 'fieldname', 'vserverid' )->first();
        $fieldValue = Capsule::table('tblcustomfieldsvalues')->where('relid', $params['serviceid'])->where('fieldid', $res->id)->first();
        $return = $clientApi->post("api/public/vps/unsuspend/{$fieldValue->value}");
        logModuleCall(
            'gridcp',
            __FUNCTION__,
            $params,
            $return
        );
        if($return['success'])
            return 'success';
        else
            return $return['message'];

    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'gridcp',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        return $e->getMessage();
    }
}

/**
 * Terminate instance of a product/service.
 *
 * @param array $params common module parameters
 *
 * @see https://developers.whmcs.com/provisioning-modules/module-parameters/
 *
 * @return string "success" or an error message
 */
function gridcp_TerminateAccount(array $params)
{
    try {
        $clientApi = new clientApi($params['serverhostname'], $params['serverusername'], $params['serverpassword'], 80, true);
        if(!$clientApi->login())
            throw new Exception('Error connecting to the GridCP panel (check server configuration)');
        if(empty($clientApi->token))
            throw new Exception('Error token.');

        $res = Capsule::table( 'tblcustomfields' )->where( 'relid', $params['pid'] )->where( 'fieldname', 'vserverid' )->first();
        $fieldValue = Capsule::table('tblcustomfieldsvalues')->where('relid', $params['serviceid'])->where('fieldid', $res->id)->first();
        $return = $clientApi->delete("api/public/vps/{$fieldValue->value}");
        logModuleCall(
            'gridcp',
            __FUNCTION__,
            $params,
            $return
        );
        if($return['success'])
            return 'success';
        else
            return $return['message'];
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'gridcp',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        return $e->getMessage();
    }

    return 'success';
}

/**
 * Change the password for an instance of a product/service.
 *
 * @param array $params common module parameters
 *
 * @see https://developers.whmcs.com/provisioning-modules/module-parameters/
 *
 * @return string "success" or an error message
 */
function gridcp_ChangePassword(array $params)
{
    try {

    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'gridcp',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        return $e->getMessage();
    }

    return 'success';
}

/**
 * Upgrade or downgrade an instance of a product/service.
 *
 * @param array $params common module parameters
 *
 * @see https://developers.whmcs.com/provisioning-modules/module-parameters/
 *
 * @return string "success" or an error message
 */
function gridcp_ChangePackage(array $params)
{
    try {

    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'gridcp',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        return $e->getMessage();
    }

    return 'success';
}

/**
 * Test connection with the given server parameters.
 *
 * @param array $params common module parameters
 *
 * @see https://developers.whmcs.com/provisioning-modules/module-parameters/
 *
 * @return array
 */
function gridcp_TestConnection(array $params)
{
    try {
        $success = false;
        $clientApi = new clientApi($params['serverhostname'], $params['serverusername'], $params['serverpassword'], 80, true);
        $result = $clientApi->login();
        if($result)
            $success = true;
        else
            throw new Exception("Error");
            //throw new Exception('Error token.');
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'gridcp',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        $success = false;
        $errorMsg = $e->getMessage();
    }

    return array(
        'success' => $success,
        'error' => $errorMsg,
    );
}

/**
 * Additional actions an admin user can invoke.
 *
 * Define additional actions that an admin user can perform for an
 * instance of a product/service.
 *
 * @see gridcp_buttonOneFunction()
 *
 * @return array
 */
function gridcp_AdminCustomButtonArray()
{
    /*return array(
        "Button 1 Display Value" => "buttonOneFunction",
        "Button 2 Display Value" => "buttonTwoFunction",
    );*/
}

/**
 * Additional actions a client user can invoke.
 *
 * Define additional actions a client user can perform for an instance of a
 * product/service.
 *
 * Any actions you define here will be automatically displayed in the available
 * list of actions within the client area.
 *
 * @return array
 */
function gridcp_ClientAreaCustomButtonArray()
{
    return array(
        "Login Panel" => "actionOneFunction"
    );
}

/**
 * Custom function for performing an additional action.
 *
 * @param array $params common module parameters
 *
 * @see https://developers.whmcs.com/provisioning-modules/module-parameters/
 * @see gridcp_AdminCustomButtonArray()
 *
 * @return string "success" or an error message
 */
function gridcp_buttonOneFunction(array $params)
{
    try {

    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'gridcp',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        return $e->getMessage();
    }

    return 'success';
}

/**
 * Custom function for performing an additional action.
 *
 * @param array $params common module parameters
 *
 * @see https://developers.whmcs.com/provisioning-modules/module-parameters/
 * @see gridcp_ClientAreaCustomButtonArray()
 *
 * @return string "success" or an error message
 */
function gridcp_actionOneFunction(array $params)
{
    try {
        $clientApi = new clientApi($params['serverhostname'], $params['serverusername'], $params['serverpassword'], 80, true);
        if(!$clientApi->login())
            throw new Exception('Error connecting to the GridCP panel (check server configuration)');

        $token = $clientApi->get('api/public/logintoken/'.$params['username'])['message'];

        return array(
            'success' => true,
            'redirectTo' => "https://{$params['serverhostname']}/login.php?logintoken={$token}",
        );
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'gridcp',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        return $e->getMessage();
    }

}

/**
 * Admin services tab additional fields.
 *
 * @param array $params common module parameters
 *
 * @see https://developers.whmcs.com/provisioning-modules/module-parameters/
 * @see gridcp_AdminServicesTabFieldsSave()
 *
 * @return array
 */
function gridcp_AdminServicesTabFields(array $params)
{
    try {
        $response = array();

        /*return array(
            'Something Editable' => '<input type="hidden" name="gridcp_original_uniquefieldname" '
                . 'value="' . htmlspecialchars($response['textvalue']) . '" />'
                . '<input type="text" name="gridcp_uniquefieldname"'
                . 'value="' . htmlspecialchars($response['textvalue']) . '" />',
        );*/


        $response = array();
        $id_server = trim(explode('-',$params["configoption1"])[0]);
        $server_select=array();
        foreach ( Capsule::table( 'tblservers' )->where( 'type', 'gridcp' )->where('id', $id_server)->get() as $row ) {
            $server_select['id'] = $row->id;
            $server_select['hostname'] = $row->hostname;
            $server_select['username'] = $row->username;
            $server_select['password'] = $row->password;

        }
        /*$clientApi = new clientApi($server_select['hostname'], $server_select['username'], decrypt($server_select['password']), 80, true);
        $clientApi->login();

        $token = $clientApi->get('api/public/logintoken/'.$params['username'])['message'];*/

        $clientApi = new clientApi($params['serverhostname'], $params['serverusername'], $params['serverpassword'], 80, true);
        if(!$clientApi->login())
            throw new Exception('Error connecting to the GridCP panel (check server configuration)');
        $token = $clientApi->get('api/public/logintoken/'.$params['username'])['message'];
        $customField = $params['customfields'];
        $params_post['external_id'] = $customField['vserverid'];
        $extraVariable1 = "https://{$params['serverhostname']}/login.php?logintoken={$token}";
        $extraVariable2 = "https://{$params['serverhostname']}/login.php?logintoken={$token}&external_id={$params_post['external_id']}&novnc";
        $response = $clientApi->get('api/public/vps/'.$params_post['external_id']);
        $vps = array();
        if($response['success'])
        {
            $vps = $response['message'];
        }
        $ips = "";
        for($i=0;count($vps['ips']) > $i;$i++)
        {
            if($i!=0)
                $ips.=', ';
            $ips.="{$vps['ips'][$i]['ip']}";
        }
        $vps['ips']=$ips;
        return array(
            '' => "<div class=\"row\">
    <div class=\"col-sm-5\">
        <b>Access to GridCP Panel</b>
    </div>
    <div class=\"col-sm-4\" style=\"margin-bottom: 1em\">
        <a href=\"{$extraVariable1}\" class=\"btn btn-info btn-block\">Login Panel</a>
    </div>
</div>

<div class=\"row\">
    <div class=\"col-sm-5\">
        <b>VNC</b>
    </div>
    <div class=\"col-sm-4\" style=\"margin-bottom: 1em\">
        <a href=\"{$extraVariable2}\" class=\"btn btn-warning btn-block\">VNC</a>
    </div>
</div>

<div class=\"row\">
    <div class=\"col-sm-5\"><b>Node</b></div>
    <div class=\"col-sm-7\">
        {$vps['node']}
    </div>
</div>

<div class=\"row\">
    <div class=\"col-sm-5\"><b>Hostname</b></div>
    <div class=\"col-sm-7\">
       {$vps['hostname']}
    </div>
</div>

<div class=\"row\">
    <div class=\"col-sm-5\"><b>IPs</b></div>
    <div class=\"col-sm-7\">
       {$vps['ips']}
    </div>
</div>

<div class=\"row\">
    <div class=\"col-sm-5\"><b>CPU</b></div>
    <div class=\"col-sm-7\">
       {$vps['cpu']} - {$vps['cores']} cores
    </div>
</div>

<div class=\"row\">
    <div class=\"col-sm-5\"><b>RAM</b></div>
    <div class=\"col-sm-7\">
       {$vps['ram']} MB
    </div>
</div>

<div class=\"row\">
    <div class=\"col-sm-5\"><b>Disk</b></div>
    <div class=\"col-sm-7\">
        {$vps['disk']} GB
    </div>
</div>

<div class=\"row\">
    <div class=\"col-sm-5\"><b>Traffic</b></div>
    <div class=\"col-sm-7\">
        <div class=\"progress\" style=\"text-align: center;\">
            <span>{$vps['traffic_consumed']} GB/{$vps['traffic']} GB</span>
            <div class=\"progress-bar progress-bar-striped\" role=\"progressbar\" style=\"width: {$vps['traffic_percent']}%\" aria-valuenow=\"10\" aria-valuemin=\"0\" aria-valuemax=\"100\"></div>
        </div>
    </div>
</div>

<div class=\"row\">
    <div class=\"col-sm-5\"><b>Root password</b></div>
    <div class=\"col-sm-7\">
        <i id=\"root_password\" style=\"display: none\">{$vps['root_password']}</i><a onclick=\"showPassword(this)\" href=\"#\">Show password</a>
    </div>
</div>

<script type=\"text/javascript\">
    function showPassword(element){
        $('#root_password').css('display', 'block');
        $(element).css('display', 'none')
    }
</script>"
        );
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'gridcp',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        // In an error condition, simply return no additional fields to display.
    }

    return array();
}

/**
 * Execute actions upon save of an instance of a product/service.
 *
 * @param array $params common module parameters
 *
 * @see https://developers.whmcs.com/provisioning-modules/module-parameters/
 * @see gridcp_AdminServicesTabFields()
 */
function gridcp_AdminServicesTabFieldsSave(array $params)
{
    // Fetch form submission variables.
    $originalFieldValue = isset($_REQUEST['gridcp_original_uniquefieldname'])
        ? $_REQUEST['gridcp_original_uniquefieldname']
        : '';

    $newFieldValue = isset($_REQUEST['gridcp_uniquefieldname'])
        ? $_REQUEST['gridcp_uniquefieldname']
        : '';

    // Look for a change in value to avoid making unnecessary service calls.
    if ($originalFieldValue != $newFieldValue) {
        try {
            // Call the service's function, using the values provided by WHMCS
            // in `$params`.
        } catch (Exception $e) {
            // Record the error in WHMCS's module log.
            logModuleCall(
                'gridcp',
                __FUNCTION__,
                $params,
                $e->getMessage(),
                $e->getTraceAsString()
            );

            // Otherwise, error conditions are not supported in this operation.
        }
    }
}

/**
 * Perform single sign-on for a given instance of a product/service.
 *
 * @param array $params common module parameters
 *
 * @see https://developers.whmcs.com/provisioning-modules/module-parameters/
 *
 * @return array
 */
function gridcp_ServiceSingleSignOn(array $params)
{
    try {
        $clientApi = new clientApi($params['serverhostname'], $params['serverusername'], $params['serverpassword'], 80, true);
        if(!$clientApi->login())
            throw new Exception('Error connecting to the GridCP panel (check server configuration)');
        $token = $clientApi->get('api/public/logintoken/'.$params['username'])['message'];

        return array(
            'success' => true,
            'redirectTo' => 'https://'.$params['serverhostname'].'/login.php?logintoken='.$token,
        );
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'gridcp',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        return array(
            'success' => false,
            'errorMsg' => $e->getMessage(),
        );
    }
}

/**
 * Perform single sign-on for a server.
 *
 * @param array $params common module parameters
 *
 * @see https://developers.whmcs.com/provisioning-modules/module-parameters/
 *
 * @return array
 */
function gridcp_AdminSingleSignOn(array $params)
{
    try {
        $clientApi = new clientApi($params['serverhostname'], $params['serverusername'], $params['serverpassword'], 80, true);
        if(!$clientApi->login())
            throw new Exception('Error connecting to the GridCP panel (check server configuration)');
        $token = $clientApi->get('api/public/logintoken/'.$params['serverusername'])['message'];

        return array(
            'success' => true,
            'redirectTo' => 'https://'.$params['serverhostname'].'/login.php?logintoken='.$token,
        );
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'gridcp',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        return array(
            'success' => false,
            'errorMsg' => $e->getMessage(),
        );
    }
}

/**
 * Client area output logic handling.
 *
 * @param array $params common module parameters
 *
 * @see https://developers.whmcs.com/provisioning-modules/module-parameters/
 *
 * @return array
 */
function gridcp_ClientArea(array $params)
{
    $requestedAction = isset($_REQUEST['customAction']) ? $_REQUEST['customAction'] : '';

    if ($requestedAction == 'manage') {
        $serviceAction = 'get_usage';
        $templateFile = 'templates/manage.tpl';
    } else {
        $serviceAction = 'get_stats';
        $templateFile = 'templates/overview.tpl';
    }

    try {
        $response = array();
        $id_server = trim(explode('-',$params["configoption1"])[0]);
            $server_select=array();
            foreach ( Capsule::table( 'tblservers' )->where( 'type', 'gridcp' )->where('id', $id_server)->get() as $row ) {
                $server_select['id'] = $row->id;
                $server_select['hostname'] = $row->hostname;
                $server_select['username'] = $row->username;
                $server_select['password'] = $row->password;

            }
        /*$clientApi = new clientApi($server_select['hostname'], $server_select['username'], decrypt($server_select['password']), 80, true);
        $clientApi->login();

        $token = $clientApi->get('api/public/logintoken/'.$params['username'])['message'];*/

        $clientApi = new clientApi($params['serverhostname'], $params['serverusername'], $params['serverpassword'], 80, true);
        if(!$clientApi->login())
            throw new Exception('Error connecting to the GridCP panel (check server configuration)');
        $token = $clientApi->get('api/public/logintoken/'.$params['username'])['message'];
        $customField = $params['customfields'];
        $params_post['external_id'] = $customField['vserverid'];
        $extraVariable1 = "https://{$params['serverhostname']}/login.php?logintoken={$token}";
        $extraVariable2 = "https://{$params['serverhostname']}/login.php?logintoken={$token}&external_id={$params_post['external_id']}&novnc";
        $response = $clientApi->get('api/public/vps/'.$params_post['external_id']);
        $vps_data = array();
        if($response['success'])
        {
            $vps_data = $response['message'];
        }
        $ips = "";
        for($i=0;count($vps_data['ips']) > $i;$i++)
        {
            if($i!=0)
                $ips.=', ';
            $ips.="{$vps_data['ips'][$i]['ip']}";
        }
        $vps_data['ips']=$ips;
        return array(
            'tabOverviewReplacementTemplate' => $templateFile,
            'templateVariables' => array(
                'extraVariable1' => $extraVariable1,
                'extraVariable2' => $extraVariable2,
                'vps' => $vps_data,
            ),
        );
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'gridcp',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        // In an error condition, display an error page.
        return array(
            'tabOverviewReplacementTemplate' => 'error.tpl',
            'templateVariables' => array(
                'usefulErrorHelper' => $e->getMessage(),
            ),
        );
    }
}
